import gtk
import gobject
import gtk.gdk
import gtk.glade

import vmdisks
import vminstall.paths
from vminstall.gtk.tree_view import PyTreeView as PyTreeView

class DisksView(PyTreeView):
    def __init__(self, disk_map={}):
        self._pyobj_index = 4
        self.model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING,
                                   gobject.TYPE_STRING, gobject.TYPE_STRING,
                                   gobject.TYPE_PYOBJECT)
        self.view = gtk.TreeView(self.model)
        column = gtk.TreeViewColumn(vminstall.msg.name, gtk.CellRendererText(), text=0)
        self.view.set_expander_column(column)
        self.view.append_column(column)
        column = gtk.TreeViewColumn(vminstall.msg.type, gtk.CellRendererText(), text=1)
        self.view.append_column(column)
        column = gtk.TreeViewColumn("Virtual Size", gtk.CellRendererText(), text=2)
        self.view.append_column(column)
        column = gtk.TreeViewColumn("Actual Size", gtk.CellRendererText(), text=3)
        self.view.append_column(column)
        self.set_disk_map(disk_map)
        
    def _populate_row(self, iter, map):
        vminstall.log.debug("Adding row to table: '%s'" % (map,))
        disk = map[0]
        vm = map[1]
        vsize = psize = ''
        blocks = disk.get_virtual_size()
        if blocks is not None:
            vsize = '%0.1f GB' % (blocks/(2.0*1024*1024), )
        blocks = disk.get_size()
        if blocks is not None:
            psize = '%0.1f GB' % (blocks/(2.0*1024*1024), )
        self.model.set(iter, 0, disk.get_pdev(), 1, disk.get_vdev_type_str(), 2,
                       vsize, 3, psize, 4, map)

    def set_disk_map(self, disk_map):
        maps = []
        for pdev in list(disk_map.keys()):
            maps.extend(disk_map[pdev])
        self.set(maps)


class DisksGUI(gobject.GObject):
    def __init__(self):
        self.pad = 10
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.connect('delete_event', self.close, None)
        self.window.set_border_width(self.pad)
        self.window.set_title("VM Disk Manager")
        self.window.set_default_size(800, 600)
        
        self.vbox = gtk.VBox(False, 0)
        self.window.add(self.vbox)
        
        # Tool bar
        self.toolbar = gtk.Toolbar()
        def insert_button(what, fn):
            tbi = gtk.ToolButton(what)
            tbi.connect('clicked', fn, None)
            self.toolbar.insert(tbi, -1)
            return tbi
        self.create_button = insert_button(gtk.STOCK_NEW, self.create_disk)
        self.delete_button = insert_button(gtk.STOCK_DELETE, self.delete_disk)
        self.toolbar.insert(gtk.SeparatorToolItem(), -1)
        self.manage_button = insert_button(gtk.STOCK_ADD, self.manage_disk)
        self.unmanage_button = insert_button(gtk.STOCK_REMOVE, self.unmanage_disk)
        self.toolbar.insert(gtk.SeparatorToolItem(), -1)
        insert_button(gtk.STOCK_REFRESH, self.refresh)
        self.vbox.pack_start(self.toolbar, False, True, 0)

        vpane = gtk.VPaned()
        # Main table
        self.scrolled = gtk.ScrolledWindow()
        self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.disks_view = DisksView()
        self.disks_view.view.connect('cursor-changed', self.disk_selected, None)
        self.scrolled.add_with_viewport(self.disks_view.view)
        vpane.pack1(self.scrolled, True, False)
        
        # Details area
        details = gtk.Frame("Details")
        vbox = gtk.VBox(False, 0)
        self.pdev_label = gtk.Label()
        self.pdev_label.set_use_markup(True)
        a = gtk.Alignment()
        a.add(self.pdev_label)
        vbox.pack_start(a, False, False, 0)
        self.proto_label = gtk.Label()
        self.proto_label.set_use_markup(True)
        a = gtk.Alignment()
        a.add(self.proto_label)
        vbox.pack_start(a, False, False, 0)
        label = gtk.Label("<b>Attached To:</b>")
        label.set_use_markup(True)
        a = gtk.Alignment()
        a.add(label)
        vbox.pack_start(a, False, False, 0)
        self.used_by = gtk.ListStore(gobject.TYPE_STRING)
        vbox.pack_start(gtk.TreeView(self.used_by), True, True, 0)
        a = gtk.Alignment()
        a.set_padding(self.pad, self.pad, self.pad, self.pad)
        a.add(vbox)
        details.add(a)
        vpane.pack2(details, True, False)
        self.vbox.pack_start(vpane, True, True, 0)
        vpane.set_position(self.window.get_size()[1])

        # Button bar on the bottom
        if False:
            self.button_bar = gtk.HButtonBox()
            self.refresh_button = gtk.Button("Refresh")
            self.refresh_button.connect_object('clicked', self.refresh, None)
            self.button_bar.add(self.refresh_button)
            self.del_button = gtk.Button("Delete")
            self.del_button.connect_object('clicked', self.delete_disk, None)
            self.button_bar.add(self.del_button)
            self.ok_button = gtk.Button("Close")
            self.ok_button.connect_object('clicked', self.destroy, None)
            self.button_bar.add(self.ok_button)
            self.vbox.pack_start(self.button_bar, False, True, self.pad)

        self.refresh()
        self.window.show_all()
        
    def is_deletable(self, pair):
        if not pair:
            return False
        disk = pair[0]
        vm = pair[1]
        if 'create' not in dir(disk.driver):
            return False
        return vm is None
        
    def create_disk(self, *args):
        pass  # FIXME
    def manage_disk(self, *args):
        pass  # FIXME
    def unmanage_disk(self, *args):
        (pair, row) = self.disks_view.get_selected()
        if not self.is_deletable(pair):
            return
        # FIXME: dialog?
        self.disks_view.delete_selected()
        dm = vmdisks.DiskMap()
        dm.unmanage_disk(pair[0])
    def delete_disk(self, *args):
        (pair, row) = self.disks_view.get_selected()
        if not self.is_deletable(pair):
            return
        dialog = gtk.MessageDialog(parent=self.window, flags=gtk.DIALOG_MODAL,
                                   type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO,
                                   message_format="Are you sure you want to delete this disk?")
        dialog.set_title("Delete disk?")
        result = dialog.run()
        dialog.destroy()
        if result == gtk.RESPONSE_YES:
            self.disks_view.delete_selected()
            pair[0].delete()
    def refresh(self, *args):
        dm = vmdisks.DiskMap()
        self.disks_view.set_disk_map(dm.disk_map)
        self.disk_selected(None)
        
    def disk_selected(self, tree_view, data=None):
        (pair, row) = self.disks_view.get_selected()
        if pair:
            self.refresh_buttons(pair)
            self.refresh_details(pair)
        
    def refresh_buttons(self, pair):
        deletable = self.is_deletable(pair)
        self.delete_button.set_sensitive(deletable)
        self.unmanage_button.set_sensitive(deletable)
        
    def refresh_details(self, pair):
        disk = pair[0]
        vm = pair[1]
        self.pdev_label.set_label("<b>Location:</b> %s" % (disk.get_pdev(), ))
        self.proto_label.set_label("<b>Storage Type:</b> %s" % (disk.get_vdev_type_str(), ))
        self.used_by.clear()
        if vm:
            self.used_by.append([vm.name])
            # FIXME:  why isn't disk map collapsing the same pdev together?
            print(vm.name)
        
    def destroy(self, args=None):
        gtk.main_quit()
    def close(self, w, e, d=None):
        gtk.main_quit()

def run():
    window = DisksGUI()
    gtk.main()
