# Fully virtualized guest support
#
# Initially derived from virt-inst, Copyright 2005-2006 Red Hat, Inc.
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 

import os
from .exceptions import *
from .full_virt_guest import FullVirtGuest
from .kvm_guest import KVMGuest
from .direct_kernel_boot import DirectKernelBoot
from vmdisks import Disk
from .log import log
import vminstall.advanced
from . import caps

class FullVirtKVMGuest(FullVirtGuest, KVMGuest):
    def __init__(self, options):
        KVMGuest.__init__(self, options)

        self._cdrom = None
        self.features = {
            "acpi": True,
            "pae": True,  # required to install 64 bit!
        }
        self._qemu = '/usr/bin/qemu'
        if caps.is_kvm():
            if caps.kvm_arch() == 's390x':
                self._qemu = '/usr/bin/qemu-system-s390x'
            elif caps.kvm_arch() == 'aarch64':
                self._qemu = '/usr/bin/qemu-system-aarch64'
            elif caps.kvm_arch() == 'ppc64le' or caps.kvm_arch() == 'ppc64':
                self._qemu = '/usr/bin/qemu-system-ppc64'
            if os.path.exists("/usr/bin/qemu-system-x86_64"):
                self._qemu = '/usr/bin/qemu-system-x86_64'
            elif os.path.exists("/usr/bin/qemu-kvm"):
                self._qemu = '/usr/bin/qemu-kvm'
        self.dkb = None

    def _get_features_xml(self):
        ret = []
        ret = vminstall.advanced.get_advanced_features_kvm(self.options)
        #if self.options.apic:
        #    ret.append('<apic/>')
        #for (k, v) in self.features.items():
        #    if v:
        #        ret.append('<%s/>' % k)
        return ret

    def GetConfigXml(self):
        if caps.is_kvm():
            dtype = "<domain type='kvm'>"
        else:
            dtype = "<domain type='qemu'>"
        return [ dtype,
                   self.getCommonXml(),
                 [ '<os>',
                     self._get_os_xml(),
                   '</os>',
                   '<features>',
                     self._get_features_xml(),
                   '</features>',
                   self._get_localtime_xml(),
                   self._get_action_xml(),
                   '<devices>',
                   [ '<emulator>%s</emulator>' % self._qemu ],
                     self._get_disk_xml(),
                     self._get_network_xml(),
                     self._get_serial_xml(),
                     self._get_host_devices_xml(),
                     self._get_input_xml(),
                     self._get_graphics_xml(),
                     self._get_sound_xml(),
                     self._get_misc_xml(),
                   '</devices>' ],
                 '</domain>\n' ]

    def PostInstallCheck(self):
        """Verify that the first hard disk is bootable."""
        hds = [disk for disk in self.options.disks if disk.vdevtype == Disk.DEVICE_DISK]
        if hds:
            vdev = hds[0].block_attach()
            try:
                if hds[0].has_bootsector():
                    return
            finally:
                hds[0].block_detach()
        raise VmError(VmError.E_NO_BOOTLOADER)
